@@ -6,9 +6,15 @@
  * Daniel Veillard <Daniel.Veillard@w3.org>
  */
 
+#ifdef WIN32
+#include "win32config.h"
+#else
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdarg.h>
-#include "parser.h"
+#include <libxml/parser.h>
 
 /**
  * xmlParserPrintFileInfo:
@@ -40,6 +46,7 @@ xmlParserPrintFileContext(xmlParserInputPtr input) {
     const xmlChar *cur, *base;
     int n;
 
+    if (input == NULL) return;
     cur = input->cur;
     base = input->base;
     while ((cur > base) && ((*cur == '\n') || (*cur == '\r'))) {
@@ -85,7 +92,7 @@ xmlParserError(void *ctx, const char *msg, ...)
     va_list args;
 
     input = ctxt->input;
-    if ((input->filename == NULL) && (ctxt->inputNr > 1)) {
+    if ((input != NULL) && (input->filename == NULL) && (ctxt->inputNr > 1)) {
 	cur = input;
         input = ctxt->inputTab[ctxt->inputNr - 2];
     }
@@ -123,7 +130,7 @@ xmlParserWarning(void *ctx, const char *msg, ...)
     va_list args;
 
     input = ctxt->input;
-    if ((input->filename == NULL) && (ctxt->inputNr > 1)) {
+    if ((input != NULL) && (input->filename == NULL) && (ctxt->inputNr > 1)) {
 	cur = input;
         input = ctxt->inputTab[ctxt->inputNr - 2];
     }
