@@ -654,6 +654,23 @@ xmlNanoHTTPConnectAttempt(struct in_addr ia, int port)
 	    close(s);
 	    return(-1);
     }
+
+    if ( FD_ISSET(s, &wfd) ) {
+	socklen_t len;
+	len = sizeof(status);
+	if (getsockopt(s, SOL_SOCKET, SO_ERROR, &status, &len) < 0 ) {
+	    /* Solaris error code */
+	    return (-1);
+	}
+	if ( status ) {
+	    close (s);
+	    errno = status;
+	    return (-1);
+	}
+    } else {
+	/* pbm */
+	return (-1);
+    }
     
     return(s);
 }
@@ -714,7 +731,7 @@ xmlNanoHTTPConnectHost(const char *host, int port)
  *     The contentType, if provided must be freed by the caller
  */
 
-void *
+void*
 xmlNanoHTTPOpen(const char *URL, char **contentType) {
     xmlNanoHTTPCtxtPtr ctxt;
     char buf[4096];
@@ -914,7 +931,7 @@ xmlNanoHTTPClose(void *ctx) {
  *     The contentType, if provided must be freed by the caller
  */
 
-void *
+void*
 xmlNanoHTTPMethod(const char *URL, const char *method, const char *input,
                   char **contentType, const char *headers) {
     xmlNanoHTTPCtxtPtr ctxt;
