@@ -159,7 +159,7 @@ xmlAttributePtr	    xmlAddAttributeDecl	    (xmlValidCtxtPtr ctxt,
 					     xmlDtdPtr dtd,
 					     const xmlChar *elem,
 					     const xmlChar *name,
-					     const xmlChar *prefix,
+					     const xmlChar *ns,
 					     xmlAttributeType type,
 					     xmlAttributeDefault def,
 					     const xmlChar *defaultValue,
