@@ -14,9 +14,9 @@
  * your library and includes mismatch
  */
 extern void xmlCheckVersion(int version);
-#define LIBXML_VERSION 20100
-#define LIBXML_VERSION_STRING "20100"
-#define LIBXML_TEST_VERSION xmlCheckVersion(20100);
+#define LIBXML_VERSION 20101
+#define LIBXML_VERSION_STRING "20101"
+#define LIBXML_TEST_VERSION xmlCheckVersion(20101);
 
 /*
  * Whether the FTP support is configured in
@@ -54,6 +54,15 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_XPATH_DISABLED
 #endif
 
+/*
+ * Whether iconv support is available
+ */
+#if 1
+#define LIBXML_ICONV_ENABLED
+#else
+#define LIBXML_ICONV_DISABLED
+#endif
+
 /*
  * Whether Debugging module is configured in
  */
